# (c) 2013-2014 The MITRE Corporation. ALL RIGHTS RESERVED.
# Contributions by Sam Cornwell, John Butterworth, Corey Kallenberg, and Xeno Kovah


import glob
import os
from collections import defaultdict

class CsvData(object):
    def __init__(self, values, filename=None):
        self.values = values
        self.filename = filename

class Version(object):
    def __init__(self, values):
        self.product_name = values.get('PRODUCT_NAME', '').strip()
        self.bios_vendor = values.get('BIOS_VENDOR', '').strip()
        self.bios_version = values.get('BIOS_VERSION', '').strip()
        self.tpm_vendor = values.get('TPM_VENDORID', '').strip()
        
    def __cmp__(self, other):
        if self.bios_vendor != other.bios_vendor:
            return cmp(self.bios_vendor, other.bios_vendor)
        elif self.product_name != other.product_name:
            return cmp(self.product_name, other.product_name)
        elif self.tpm_vendor != other.tpm_vendor:
            return cmp(self.tpm_vendor, other.tpm_vendor)
        else:
            return cmp(self.bios_version, other.bios_version)
    
    def __eq__(self, other):
        return cmp(self, other) == 0
    
    def __keys(self):
        return (self.bios_vendor, self.product_name, self.bios_version, self.tpm_vendor)
        
    def __hash__(self):
        return hash(self.__keys())

def parse_all_csvs(directories, data_transform=lambda *args: args, verbose = False):
    csv_directories = [dir + os.sep if dir[-1] != os.sep else dir for dir in directories]
    filenames = sum([glob.glob(dir + '*.csv') for dir in csv_directories], [])
    
    data = {}
    
    for filename in filenames:
        try:
            with open(filename, 'r') as f:
                values = csv_to_dict(f.read())
            data[filename] = data_transform(values, filename)
        except Exception, e:
            # completely ignore any files we couldn't parse
            if verbose:
                print "Failed to parse", filename
    return data

def csv_to_dict(data):
    d = {}
    
    for field in data.split(','):
        try:
            (key, value) = field.split('=', 1)
            d[key] = value
        except:
            # ignore this field I guess
            pass

    return d

def group_by_version(csvdata_list):
    groups = defaultdict(list)
    
    for csvdata in csvdata_list:
        groups[Version(csvdata.values)].append(csvdata)
    
    return groups
